<?php
/**
 * Plugin Name: Product Price by Formula Pro for WooCommerce
 * Plugin URI: https://www.persianscript.ir
 * Description: Set formula for automatic WooCommerce product price calculation.
 * Version: 2.4.0
 * Author: پرشین اسکریپت
 * Author URI: https://www.persianscript.ir
 * Text Domain: product-price-by-formula-pro-for-woocommerce
 * Domain Path: /langs
 * Copyright: © 2023 ProWCPlugins.com
 * WC tested up to: 7.6
 * License: GNU General Public License v3.0
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

define( 'PPBF_URL', plugin_dir_url( __FILE__ ) );
define( 'PPBF_STORE_URL', 'https://prowcplugins.com' );
define( 'PPBF_TEXTDOMAIN', 'product-price-by-formula-pro-for-woocommerce' );
define( 'PPBF_SITE', home_url());

if ( ! class_exists( 'Prowc_PPBF' ) ) :

/**
 * Main Prowc_PPBF Class
 *
 * @class   Prowc_PPBF
 * @version 2.3.2
 * @since   1.0.0
 */
final class Prowc_PPBF {

	/**
	 * Plugin version.
	 *
	 * @var   string
	 * @since 1.0.0
	 */
	public $version = '2.4.0';

	/**
	 * @var   Prowc_PPBF The single instance of the class
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * Main Prowc_PPBF Instance
	 *
	 * Ensures only one instance of Prowc_PPBF is loaded or can be loaded.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @static
	 * @return  Prowc_PPBF - Main instance
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Prowc_PPBF Constructor.
	 *
	 * @version 2.2.0
	 * @since   1.0.0
	 * @access  public
	 */
	function __construct() {

		// Set up localisation
		load_plugin_textdomain( 'product-price-by-formula-pro-for-woocommerce', false, dirname( plugin_basename( __FILE__ ) ) . '/langs/' );

		// The filter
		add_filter( 'prowc_ppbf', array( $this, 'prowc_ppbf' ), PHP_INT_MAX, 3 );

		// Include required files
		$this->includes();

		// Admin
		if ( is_admin() ) {
			$this->admin();
		}
	}

	/**
	 * prowc_ppbf.
	 *
	 * @version 2.3.2
	 * @since   1.0.0
	 * @todo    [dev] (now) `if_product_taxonomy`: check if `array_intersect` is safe to use (in older PHP versions)
	 * @todo    [dev] `if_product_taxonomy`: add `slug_starts_with` and `not_slug_starts_with` (#13002)
	 * @todo    [dev] `if_product_title` (with `equals`, `not_equals`, `starts_with` and `not_starts_with`) (maybe as part of `[if_value]`) (#13002)
	 * @todo    [dev] `if_value`: `in`, `not_in`: add `sep` and `do_trim` atts
	 * @todo    [dev] (maybe) `if_type_price`, `if_current_filter`: `evaluate_formula` on `content`?
	 * @todo    [dev] (maybe) `if_current_filter`: filter `priority`?
	 * @todo    [dev] (maybe) `if_product_taxonomy`: `multiple` should default to `yes`?
	 */
	function prowc_ppbf( $value, $type, $args = array() ) {
		switch ( $type ) {
			case 'settings':
				return '';
			case 'value_enable_for_all_products':
				return get_option( 'prowc_ppbf_enable_for_all_products', 'no' );
			case 'value_same_formula_for_all_products':
				return get_option( 'prowc_ppbf_same_formula_for_all_products', 'no' );
			case 'if_product_taxonomy':
				if ( ! isset( $args['atts']['taxonomy'] ) ) {
					return '';
				}
				$product_id = ( isset( $args['atts']['use_parent_product'] ) && 'yes' === $args['atts']['use_parent_product'] &&
					0 != ( $parent_product_id = wp_get_post_parent_id( $args['product_id'] ) ) ?
						$parent_product_id : $args['product_id'] );
				if ( ! isset( $args['atts']['multiple'] ) && 'no' === $args['atts']['multiple'] ) {
					$product_term_slug = false;
					$product_terms     = get_the_terms( $product_id, $args['atts']['taxonomy'] );
					if ( $product_terms && ! is_wp_error( $product_terms ) ) {
						foreach ( $product_terms as $product_term ) {
							$product_term_slug = $product_term->slug;
							break;
						}
					}
					return (
						( ! empty( $args['atts']['slug'] )     && ( ! $product_term_slug || ! in_array( $product_term_slug, array_map( 'trim', explode( ',', $args['atts']['slug'] ) ) ) ) ) ||
						( ! empty( $args['atts']['not_slug'] ) &&     $product_term_slug &&   in_array( $product_term_slug, array_map( 'trim', explode( ',', $args['atts']['not_slug'] ) ) ) )
					) ? '' : $args['content'];
				} else { // `multiple` = `yes`
					$product_term_slugs = false;
					$product_terms      = get_the_terms( $product_id, $args['atts']['taxonomy'] );
					if ( $product_terms && ! is_wp_error( $product_terms ) ) {
						foreach ( $product_terms as $product_term ) {
							$product_term_slugs[] = $product_term->slug;
						}
					}
					return (
						( ! empty( $args['atts']['slug'] )     && ( ! $product_term_slugs ||   empty( array_intersect( $product_term_slugs, array_map( 'trim', explode( ',', $args['atts']['slug'] ) ) ) ) ) ) ||
						( ! empty( $args['atts']['not_slug'] ) &&     $product_term_slugs && ! empty( array_intersect( $product_term_slugs, array_map( 'trim', explode( ',', $args['atts']['not_slug'] ) ) ) ) )
					) ? '' : $args['content'];
				}
			case 'if_value':
				if ( ! isset( $args['atts']['value'] ) || ! isset( $args['atts']['compare_operator'] ) || ! isset( $args['atts']['compare_to_value'] ) ) {
					return '';
				}
				$value            = prowc_ppbf_process_atts_shortcodes( $args['atts']['value'] );
				$compare_to_value = prowc_ppbf_process_atts_shortcodes( $args['atts']['compare_to_value'] );
				$result           = false;
				switch ( $args['atts']['compare_operator'] ) {
					case 'equal':
						$result = ( $value == $compare_to_value );
						break;
					case 'not_equal':
						$result = ( $value != $compare_to_value );
						break;
					case 'less':
						$result = ( $value <  $compare_to_value );
						break;
					case 'less_or_equal':
						$result = ( $value <= $compare_to_value );
						break;
					case 'more':
					case 'greater':
						$result = ( $value >  $compare_to_value );
						break;
					case 'more_or_equal':
					case 'greater_or_equal':
						$result = ( $value >= $compare_to_value );
						break;
					case 'in':
						$compare_to_value = array_map( 'trim', explode( ',', $compare_to_value ) );
						$result = in_array( $value, $compare_to_value );
						break;
					case 'not_in':
						$compare_to_value = array_map( 'trim', explode( ',', $compare_to_value ) );
						$result = ! in_array( $value, $compare_to_value );
						break;
					case 'between':
						$compare_to_value = array_map( 'trim', explode( ',', $compare_to_value ) );
						$result = ( ! isset( $args['atts']['inclusive'] ) || 'yes' == $args['atts']['inclusive'] ?
							( 2 == count( $compare_to_value ) && $value >= $compare_to_value[0] && $value <= $compare_to_value[1] ) :
							( 2 == count( $compare_to_value ) && $value >  $compare_to_value[0] && $value <  $compare_to_value[1] ) );
						break;
					case 'not_between':
						$compare_to_value = array_map( 'trim', explode( ',', $compare_to_value ) );
						$result = ( ! isset( $args['atts']['inclusive'] ) || 'yes' == $args['atts']['inclusive'] ?
							( 2 == count( $compare_to_value ) && ( $value <  $compare_to_value[0] || $value >  $compare_to_value[1] ) ) :
							( 2 == count( $compare_to_value ) && ( $value <= $compare_to_value[0] || $value >= $compare_to_value[1] ) ) );
						break;
				}
				return ( $result ? do_shortcode( $args['content'] ) : '' );
			case 'if_type_price':
				$current_filter = $args['current_filter'];
				$price_type     = $args['price_type'];
				$content        = $args['content'];
				if ( prowc_ppbf()->core->current_product->is_type( 'variation' ) ) {
					$price_filters      = array( ( 'regular_price' === $price_type ? 'woocommerce_variation_prices_regular_price'     : 'woocommerce_variation_prices_sale_price' ) );
					$main_price_filters = array( 'woocommerce_variation_prices_price' );
					if ( ! prowc_ppbf()->core->is_wc_version_below_3 ) {
						$price_filters[]      = ( 'regular_price' === $price_type ? 'woocommerce_product_variation_get_regular_price' : 'woocommerce_product_variation_get_sale_price' );
						$main_price_filters[] = 'woocommerce_product_variation_get_price';
					}
				} else {
					$price_filters      = array( prowc_ppbf()->core->get_price_filter( ( 'regular_price' === $price_type ? 'regular_price' : 'sale_price' ) ) );
					$main_price_filters = array( prowc_ppbf()->core->get_price_filter( 'price' ) );
				}
				return (
						( in_array( $current_filter, $price_filters ) ) ||
						( in_array( $current_filter, $main_price_filters ) && (
							( 'regular_price' === $price_type && ! prowc_ppbf()->core->current_product->is_on_sale() ) ||
							( 'sale_price'    === $price_type &&   prowc_ppbf()->core->current_product->is_on_sale() ) ) )
					? do_shortcode( $content ) : '' );
			case 'if_current_filter':
				return ( $args['atts']['current_filter'] === $args['current_filter'] ? do_shortcode( $args['content'] ) : '' );
		}
		return $value;
	}

	/**
	 * Include required core files used in admin and on the frontend.
	 *
	 * @version 2.3.2
	 * @since   1.0.0
	 */
	function includes() {
		// Lib
		require_once( 'includes/lib/PHPMathParser/Math.php' );
		// Functions
		require_once( 'includes/prowc-ppbf-functions.php' );
		// Core
		$this->core = require_once( 'includes/class-prowc-ppbf-core.php' );
	}

	/**
	 * admin side add actions
	 *
	 * @version 2.2.0
	 * @since   2.2.0
	 */
	function admin() {

		//update database
		add_action('plugins_loaded', array($this,'prowc_product_price_by_formula_pro_check_version'));

		// Action links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'action_links' ) );
		// Settings
		add_filter( 'woocommerce_get_settings_pages', array( $this, 'add_woocommerce_settings_tab' ) );
		require_once( 'includes/settings/class-prowc-ppbf-metaboxes.php' );
		require_once( 'includes/settings/class-prowc-ppbf-settings-section.php' );
		require_once( 'includes/ppbf-license.php' );
		
		$this->settings = array();
		$this->settings['general']           = require_once( 'includes/settings/class-prowc-ppbf-settings-general.php' );
		$this->settings['default_formula']   = require_once( 'includes/settings/class-prowc-ppbf-settings-default-formula.php' );
		// Version updated
		if ( get_option( 'prowc_ppbf_version', '' ) !== $this->version ) {
			add_action( 'admin_init', array( $this, 'version_updated' ) );
		}

		// Admin notice
		if (!class_exists('WooCommerce')) {
			add_action('admin_notices', array( $this, 'fail_load') );
			return;
		}
	}

	/**
	 * Database options upgrade
	 *
	 * @version 2.3.4
	 */
	function prowc_product_price_by_formula_pro_check_version() {
		
		if ( version_compare( $this->version , '2.3.3', '>' )) { 
			global $wpdb;
			$table_options = $wpdb->prefix.'options';
			$old_keys = $wpdb->get_results( "SELECT *  FROM `". $table_options ."` WHERE `option_name` LIKE '%alg_wc_%'");
			if (is_array($old_keys) || is_object($old_keys)){
				foreach($old_keys as $val) {
					$new_key = str_replace("alg_wc_","prowc_", $val->option_name);
					$wpdb->query( $wpdb->prepare( "UPDATE $table_options SET option_name = %s WHERE option_name = %s", $new_key, $val->option_name ) );
				}
			}
		}
	}
	
	/**
	 * Show action links on the plugin screen.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @param   mixed $links
	 * @return  array
	 */
	function action_links( $links ) {
		$custom_links = array();
		$custom_links[] = '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=prowc_ppbf' ) . '">' . __( 'Settings', 'woocommerce' ) . '</a>';
		if ( 'product-price-by-formula-pro-for-woocommerce.php' === basename( __FILE__ ) ) {
			$custom_links[] = '<a href="https://prowcplugins.com/downloads/product-price-by-formula-pro-for-woocommerce/">' .
				__( 'Unlock All', 'product-price-by-formula-pro-for-woocommerce' ) . '</a>';
		}
		return array_merge( $custom_links, $links );
	}

	/**
	 * Add Product Price by Formula settings tab to WooCommerce settings.
	 *
	 * @version 2.0.0
	 * @since   1.0.0
	 */
	function add_woocommerce_settings_tab( $settings ) {
		$settings[] = require_once( 'includes/settings/class-prowc-settings-ppbf.php' );
		return $settings;
	}

	/**
	 * version_updated.
	 *
	 * @version 2.2.0
	 * @since   1.1.0
	 */
	function version_updated() {
		update_option( 'prowc_ppbf_version', $this->version );
	}

	/**
	 * Get the plugin url.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_url() {
		return untrailingslashit( plugin_dir_url( __FILE__ ) );
	}

	/**
	 * Get the plugin path.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

	/**
	 * Admin Notice for WooCommerce Install & Active.
	 *
	 * @version 2.3.3
	 * @since   2.3.3
	 * @return  string
	 */
	function prowc_ppbf_installed() {

		$file_path = 'woocommerce/woocommerce.php';
		$installed_plugins = get_plugins();

		return isset($installed_plugins[$file_path]);
	}

	/**
	 * Admin Notice for WooCommerce Install & Active.
	 *
	 * @version 2.3.3
	 * @since   2.3.3
	 * @return  string
	 */
	function fail_load() {
		if(function_exists('WC')){
			return;
		}
	    $screen = get_current_screen();
	    if (isset($screen->parent_file) && 'plugins.php' === $screen->parent_file && 'update' === $screen->id) {
	        return;
	    }

	    $plugin = 'woocommerce/woocommerce.php';
	    if ($this->prowc_ppbf_installed()) {
	        if (!current_user_can('activate_plugins')) {
	            return;
	        }
	        $activation_url = wp_nonce_url('plugins.php?action=activate&amp;plugin=' . $plugin . '&amp;plugin_status=all&amp;paged=1&amp;s', 'activate-plugin_' . $plugin);

	        $message = '<p><strong>' . esc_html__('Product Price by Formula Pro for WooCommerce', PPBF_TEXTDOMAIN) . '</strong>' . esc_html__(' plugin is not working because you need to activate the Woocommerce plugin.', PPBF_TEXTDOMAIN) . '</p>';
	        $message .= '<p>' . sprintf('<a href="%s" class="button-primary">%s</a>', $activation_url, __('Activate Woocommerce Now', PPBF_TEXTDOMAIN)) . '</p>';
	    } else {
	        if (!current_user_can('install_plugins')) {
	            return;
	        }

	        $install_url = wp_nonce_url(self_admin_url('update.php?action=install-plugin&plugin=Woocommerce'), 'install-plugin_Woocommerce');

	        $message = '<p><strong>' . esc_html__('Product Price by Formula Pro for WooCommerce', PPBF_TEXTDOMAIN) . '</strong>' . esc_html__(' plugin is not working because you need to install the WooCoomerce plugin', PPBF_TEXTDOMAIN) . '</p>';
	        $message .= '<p>' . sprintf('<a href="%s" class="button-primary">%s</a>', $install_url, __('Install WooCoomerce Now', PPBF_TEXTDOMAIN)) . '</p>';
	    }

	    echo '<div class="error"><p>' . $message . '</p></div>';
	}

}

endif;

if ( ! function_exists( 'prowc_ppbf' ) ) {
	/**
	 * Returns the main instance of Prowc_PPBF to prevent the need to use globals.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  Prowc_PPBF
	 */
	function prowc_ppbf() {
		return Prowc_PPBF::instance();
	}
}

prowc_ppbf();
